#!/bin/sh
# test_elsa_installation.sh
#
# Copyright (C) 2005-2006 BULL SA.
# Author: Guillaume Thouvenin <guillaume.thouvenin@bull.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# 
# This script is used to validate the installation of the Enhanced Linux
# System Accounting on your system. 
# Here are the different stages of the test:
#         1) Check if proc_fork_connector function is available on your
#            system
#         2) Check if you are root because you need to be root to launch
#            the job daemon
#         3) Restart the job daemon because the job historic must be 
#            empty
#         4) Add a new process that will create a new job
#         5) Add another process in the same job
#         6) Create a new job with a new process
#         7) That's all for now...


##########################################################################
# VARIABLE

FILE_TEMP=/tmp/elsa_test_valid.tmp
LOGNAME=`id -un`

RES_COL=70
MOVE_TO_COL="\\033[${RES_COL}G"
SUCCESS=$'\e[32;01m'
FAILURE=$'\e[31;01m'
NORMAL=$'\e[0m'
BRACKET=$'\e[34;01m'

##########################################################################
# COMMAND

JOBD=/sbin/jobd
JOBMNG=/bin/jobmng
ELSA=/bin/elsa

##########################################################################
# FUNCTION

# echoing a green OK
echo_ok() {
	echo -e "${MOVE_TO_COL}${BRACKET}[ ${SUCCESS}OK${BRACKET} ]${NORMAL}"
	return 0
}

# echoing a red KO
echo_ko() {
	echo -e "${MOVE_TO_COL}${BRACKET}[ ${FAILURE}KO${BRACKET} ]${NORMAL}"
	return 0
}

# Display the title of the current test
test_title() {
        echo -ne " ${SUCCESS}*${NORMAL} ${*}..."
        return 0
}

# If the first value passed is not equal to 0, it displays a KO with a
# message given as a second argument.
test_passed() {
	if [ $1 -eq 0 ]
	then
		echo_ok
	else
		echo_ko; echo -e "\t$2"
	fi
	return $1
}

# If previous test is not passed, program exit
test_exit_if_failed() {
	test_passed $1
	if [ ! $? -eq 0 ]; then exit 0; fi
}

test_id_root() {
	return `id -u`
}


ELSA_TMP_FILE=/tmp/elsa.tmp
ACCTON=/usr/sbin/accton  # on fedora
test_bsd_acct_ver() {
	rm -f ${ELSA_TMP_FILE}
	touch ${ELSA_TMP_FILE}

	# start process accounting
	${ACCTON} ${ELSA_TMP_FILE}

	# run a command to fill the accouting file
	touch ${ELSA_TMP_FILE}

	# Now we read the 2nd bytes of the file and keep the second one
	# which is the BSD version. 'od' displays an ASCII character. It
	# means that 3 will be displayed as '003'
	RES=`head -c2 ${ELSA_TMP_FILE} | od -c | awk '{print $3}'`

	# stop process accounting
	${ACCTON}

	# remove elsa.tmp
	rm -f ${ELSA_TMP_FILE}

	# return the result
	if [ ${RES} -eq "003" ]
       	then 
		# it's OK
		return 0
	else
		# failed
		return 1
	fi
}

##########################################################################
# MAIN

# You must be root
test_title "Check if you are root"
test_id_root
test_exit_if_failed $?

# We must use BSD Process Accounting Version 3
test_title "Check if BSD Process Accounting version is 3"
test_bsd_acct_ver
test_exit_if_failed $?

# First we check if the kernel has the proc_fork_connector
test_title "Check if proc_fork_connector is in /proc/kallsysms"
cat /proc/kallsyms | grep proc_fork_connector &> /dev/null
# If found, $? contains 0.
test_exit_if_failed $?


# Must be root to start the job daemon
test_title "Check if you're root in order to (re-)start the job daemon"
test ${LOGNAME} == root
test_exit_if_failed $?


# We must check if there is a running job daemon. 
# If not, we must start a new one.
test_title "Check if there is no other job daemon"
ps aux | grep jobd | grep -v grep &> /dev/null
# if a jobd is running, the previous command will return 0.
if [ $? -eq 0 ]
then
	echo_ko
	echo -e "\tjobd is running. As job must be empty for the"
	echo -e "\ttest we must restart it. All data will be lost."
	echo -e "\tAgree [Y/n]?"
	read answer
	if [ $answer == "n" ] || [ $answer == "N" ]
	then
		echo "test halted..."
		exit 2
	else
		killall -USR1 jobd
	fi
else
	echo_ok
fi

# Start the job daemon
test_title "start the jobd daemon"
${JOBD}
test_exit_if_failed $?

# At this point, jobd is running and we can start the test
if [ -f ${FILE_TEMP} ]
then
	test_title "destroy ${FILE_TEMP}"
	rm -f ${FILE_TEMP}
	test_exit_if_failed $?
fi

# create one job with one process and check if jobID #1 
# is created abd contains one process
test_title "create a new job with one process"
${JOBMNG} -a -e 'ls' &> /dev/null
${JOBMNG} -d > ${FILE_TEMP}
COMPTEUR=`${ELSA} -d -j ${FILE_TEMP} | \
awk 'BEGIN {count=0;}
     /JOB# 1/ {count++}
     END {print count}'`
test ${COMPTEUR} -eq 1
test_passed $?; 
	
# add a new process in the same job and
# check if jobID #1 contains two processes
test_title "Add a new process in the same job"
${JOBMNG} -a -e 'ls' -j 1 &> /dev/null
${JOBMNG} -d > ${FILE_TEMP}
COMPTEUR=`${ELSA} -d -j ${FILE_TEMP} | \
awk 'BEGIN {count=0;}
     /JOB# 1/ {count++}
     END {print count}'`
test ${COMPTEUR} -eq 2
test_passed $?; 

# add a new process in new job, check if jobID #1 
# contains two processes and if jobID #2 contains 
# one process
test_title "Add a new process in a new job"
${JOBMNG} -a -e 'ls' &> /dev/null
${JOBMNG} -d > ${FILE_TEMP}
COMPTEUR=`${ELSA} -d -j ${FILE_TEMP} | \
awk 'BEGIN {count=0;}
     /JOB# 1/ {count++}
     END {print count}'`
if [ ${COMPTEUR} -eq 2 ]
then
	COMPTEUR=`${ELSA} -d -j ${FILE_TEMP} | \
	awk 'BEGIN {count=0;}
     	     /JOB# 2/ {count++}
     	     END {print count}'`
	test  ${COMPTEUR} -eq 1
	test_passed $?; 
else 
	test_passed 1
fi

# test are finished, we can stop the job daemon
test_title "stop the job daemon"
${JOBMNG} -s
test_passed $?

if [ -f ${FILE_TEMP} ]
then
	test_title "destroy ${FILE_TEMP}"
	rm -f ${FILE_TEMP}
	test_passed $?
fi

# All done, exit OK
exit 0
